package live

import (
	"errors"
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/golangkit/formatime"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"live/app/lib"
	"live/app/lib/recook"
	"live/app/lib/tencent"
	"live/app/logic/goods"
	live2 "live/app/model/live"
	"live/app/model/user"
	"log"
	"strconv"
	"time"
)

type live struct {
}

var LiveLogic = &live{}

// @Title 直播列表
func (l *live) Lists(page lib.Page) []live2.LiveItem {
	liveItem := &live2.LiveItem{}
	start := (page.GetPage() - 1) * page.GetLimit()
	return liveItem.Lists(start, page.GetLimit())
}

type LiveInfo struct {
	LiveItemId uint   `json:"liveItemId"`
	PushUrl    string `json:"pushUrl"`
	GroupId    string `json:"groupId"`
}

// @Title 开播
func (l live) Start(userId uint, title, cover string, topic uint, goodsIds []uint) (result LiveInfo, err error) {
	liveModel := &live2.LiveRoom{}
	liveRoom := liveModel.GetLiveByUserId(userId)
	if liveRoom.Id <= 0 {
		// 创建直播聊天群
		err, info := ImLogic.LoginUserInfo(userId)
		if err != nil {
			return LiveInfo{}, err
		}
		groupId := fmt.Sprintf("group_%d", userId)
		if err = tencent.Im.AddLiveGroup(info.Identifier, "直播群", groupId); err != nil {
			log.Println(err)
			return LiveInfo{}, err
		}
		liveRoom = live2.LiveRoom{
			UserId:     userId,
			StreamName: fmt.Sprintf("recook_%v", userId),
			GroupId:    groupId,
		}
		liveModel.Create(&liveRoom)
	} else {
		err, info := ImLogic.LoginUserInfo(userId)
		if err != nil {
			return LiveInfo{}, err
		}
		if err = tencent.Im.AddLiveGroup(info.Identifier, "直播群", liveRoom.GroupId); err != nil {
		}
	}
	userAllFins := (&user.UserFollow{}).GetCountAll(userId)
	liveItemModel := &live2.LiveItem{}
	liveItemId := uint(0)
	if err = liveItemModel.Transaction(func(tx *gorm.DB) error {
		liveItemModel.SetDb(tx)
		liveItem := live2.LiveItem{
			UserId:      userId,
			RoomId:      liveRoom.Id,
			MainGoodsId: goodsIds[0],
			Title:       title,
			Cover:       cover,
			Topic:       topic,
			Status:      1,
			StartAt:     formatime.NewSecondNow(),
			Fans:        uint(userAllFins),
		}
		liveItemModel.Create(&liveItem)
		if liveItem.Id <= 0 {
			return errors.New("网络异常")
		}
		liveItemId = liveItem.Id

		liveGoodsModel := &live2.LiveGoods{}
		liveGoodsModel.SetDb(tx)
		liveGoodsList := []live2.LiveGoods{}
		for _, goodsId := range goodsIds {
			liveGoodsList = append(liveGoodsList, live2.LiveGoods{
				UserId:     userId,
				GoodsId:    goodsId,
				LiveItemId: liveItem.Id,
			})
		}
		row := liveGoodsModel.CreateAll(&liveGoodsList)
		if row <= 0 {
			return errors.New("网络异常")
		}

		// 加入橱窗
		(&goods.Cupboard{}).Add(userId, goodsIds)

		liveDataModel := &live2.LiveData{}
		liveData := live2.LiveData{
			UserId:     userId,
			LiveItemId: liveItem.Id,
		}
		liveDataModel.Create(&liveData)
		if liveData.Id <= 0 {
			return errors.New("网络异常")
		}
		return nil
	}); err != nil {
		return
	}

	result.LiveItemId = liveItemId
	result.PushUrl = tencent.Live.GetPushUrl(liveRoom.StreamName, liveItemId)
	result.GroupId = liveRoom.GroupId
	return
}

// @Title 结束直播
func (l *live) Stop(userId, liveItemId uint) error {
	// 获取直播场次信息
	liveItemModel := &live2.LiveItem{}
	liveItem := liveItemModel.FindById(liveItemId)
	if liveItem.Id <= 0 || liveItem.UserId != userId {
		return errors.New("参数错误")
	}

	// 获取直播期间订单统计信息
	liveOrderData, err := recook.Order.LiveOrderData(userId, liveItem.Id)
	if err != nil {
		return errors.New("网络异常")
	}
	salesVolume, _ := decimal.NewFromString((*liveOrderData).AmountSum)
	anticipatedRevenue, _ := decimal.NewFromString((*liveOrderData).AmountSum)
	buy, _ := strconv.Atoi((*liveOrderData).AmountSum)

	// 获取直播期间粉丝，时长统计信息
	endTime := formatime.NewSecondNow()
	userAllFins := (&user.UserFollow{}).GetCountAll(userId)
	duration := int(endTime.Time.Unix() - liveItem.StartAt.Time.Unix())

	// 更新数据
	liveDataModel := &live2.LiveData{}
	liveData := liveDataModel.GetByLiveItemId(liveItemId)

	return liveItemModel.Transaction(func(tx *gorm.DB) error {
		// 完成直播
		liveItemModel.SetDb(tx)
		liveItemModel.UpdateById(liveItem.Id, map[string]interface{}{
			"status": live2.LIVE_STATUS_finish,
			"end_at": endTime,
		})

		// 完成统计信息
		liveDataModel.SetDb(tx)
		liveDataModel.UpdateByLiveItemId(liveItem.Id, &live2.LiveData{
			SalesVolume:        salesVolume,
			AnticipatedRevenue: anticipatedRevenue,
			Buy:                buy,
			Duration:           duration,
			Fans:               int(uint(userAllFins) - liveItem.Fans),
		})

		ImLogic.SendLiveGroupMessage(liveItem.RoomId, LiveGroupMsgTypeLiveStop, gin.H{
			"liveItemId": liveItem.Id,
			"praise":     liveData.Praise,
			"look":       liveData.Look,
		})
		// 解散直播群
		liveRoom := live2.LiveRoom{}
		userLiveRoom := liveRoom.GetLiveById(liveItem.RoomId)
		if err := tencent.Im.DelLiveGroup(userLiveRoom.GroupId); err != nil {
			log.Println(err)
		}
		return nil
	})
}

// @Title 直播视频录制
func (l *live) Transcribe(liveItemId uint) error {
	liveItem := &live2.LiveItem{}
	item := liveItem.FindById(liveItemId)
	if item.TranscribeType != live2.LIVE_Transcribe_Type_none {
		return nil
	}
	if item.FileId == "" {
		row := liveItem.Transcribe(liveItemId, live2.LIVE_Transcribe_Type_ing)
		if row <= 0 {
			return errors.New("网络错误")
		}
	} else {
		row := liveItem.Transcribe(liveItemId, live2.LIVE_Transcribe_Type_finish)
		if row <= 0 {
			return errors.New("网络错误")
		}
	}
	return nil
}

type replyStopLiveInfo struct {
	Nickname           string          `json:"nickname"`
	HeadImgUrl         string          `json:"headImgUrl"`
	Duration           int             `json:"duration"`
	SalesVolume        decimal.Decimal `json:"salesVolume"`
	MonthDuration      int             `json:"monthDuration"`
	AnticipatedRevenue decimal.Decimal `json:"anticipatedRevenue"`
	Buy                int             `json:"buy"`
	Look               int             `json:"look"`
	Praise             int             `json:"praise"`
	Fans               int             `json:"'fans'"`
}

// @Title 获取直播场次统计数据
func (l *live) GetLiveInfo(userId, liveItemId uint) replyStopLiveInfo {
	liveDataModel := &live2.LiveData{}
	liveData := liveDataModel.GetByLiveItemId(liveItemId)

	users, _ := recook.User.GetUsers([]uint{userId})
	userItem := recook.ReplyUserItem{}
	if len(*users) > 0 {
		userItem = (*users)[0]
	}
	// 获取月直播时长
	liveItemModel := &live2.LiveItem{}
	startTime, _ := time.ParseInLocation("2006-01", time.Now().Format("2006-01"), time.Local)
	liveItems := liveItemModel.GetListByTime(userId, formatime.NewSecondFrom(startTime), formatime.NewSecondNow())
	liveItemIds := []uint{}
	for _, item := range liveItems {
		liveItemIds = append(liveItemIds, item.Id)
	}

	sumDuration := liveDataModel.SumDuration(liveItemIds)
	return replyStopLiveInfo{
		Nickname:           userItem.Nickname,
		HeadImgUrl:         userItem.HeadImgUrl,
		Duration:           liveData.Duration,
		MonthDuration:      sumDuration,
		SalesVolume:        liveData.SalesVolume,
		AnticipatedRevenue: liveData.AnticipatedRevenue,
		Buy:                liveData.Buy,
		Look:               liveData.Look,
		Praise:             liveData.Praise,
		Fans:               liveData.Fans,
	}
}
